# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V15::Enums::AdvertisingChannelTypeEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED     => "UNSPECIFIED",
  UNKNOWN         => "UNKNOWN",
  SEARCH          => "SEARCH",
  DISPLAY         => "DISPLAY",
  SHOPPING        => "SHOPPING",
  HOTEL           => "HOTEL",
  VIDEO           => "VIDEO",
  MULTI_CHANNEL   => "MULTI_CHANNEL",
  LOCAL           => "LOCAL",
  SMART           => "SMART",
  PERFORMANCE_MAX => "PERFORMANCE_MAX",
  LOCAL_SERVICES  => "LOCAL_SERVICES",
  DISCOVERY       => "DISCOVERY",
  TRAVEL          => "TRAVEL"
];

1;
