# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V17::Enums::CallToActionTypeEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED => "UNSPECIFIED",
  UNKNOWN     => "UNKNOWN",
  LEARN_MORE  => "LEARN_MORE",
  GET_QUOTE   => "GET_QUOTE",
  APPLY_NOW   => "APPLY_NOW",
  SIGN_UP     => "SIGN_UP",
  CONTACT_US  => "CONTACT_US",
  SUBSCRIBE   => "SUBSCRIBE",
  DOWNLOAD    => "DOWNLOAD",
  BOOK_NOW    => "BOOK_NOW",
  SHOP_NOW    => "SHOP_NOW",
  BUY_NOW     => "BUY_NOW",
  DONATE_NOW  => "DONATE_NOW",
  ORDER_NOW   => "ORDER_NOW",
  PLAY_NOW    => "PLAY_NOW",
  SEE_MORE    => "SEE_MORE",
  START_NOW   => "START_NOW",
  VISIT_SITE  => "VISIT_SITE",
  WATCH_NOW   => "WATCH_NOW"
];

1;
